//==============================================================================
//!	@file   my_go.h
//!	@brief  囲碁AI
//==============================================================================

#pragma once

//==============================================================================
//  Include
//==============================================================================
#include "stdafx.h"

//==============================================================================
//  Code
//==============================================================================

namespace my_go {
	
//--------------------------------------
//! @brief  エンジンクラス
//--------------------------------------
class engine : public ::engine {
public:
	class Impl;
private:
	Impl *m_pImpl;
public:
	//--------------------------------------
	//! @brief  コンストラクタ
	//--------------------------------------
	engine();
	
	//--------------------------------------
	//! @brief  デストラクタ
	//--------------------------------------
	virtual ~engine();

	//--------------------------------------
	//! @brief  リセット
	//	void engine::reset(int size, float komi)
	//	GTPの boardsize, komi, clear_board コマンドに対応しています。
	//	指定のサイズ、KOMIで盤情報をリセットして下さい。
	//	boardsize, komi, clear_boardが呼ばれる度に実行されますので、繰り返し実
	//	行されることに注意して下さい。
	//--------------------------------------
	virtual void reset(int size, float komi) ;
	
	//--------------------------------------
	//! @brief  デバッグ出力
	//--------------------------------------
//	void debugprint(char *str, ...);
//	void dumpboard() ;
	
	//--------------------------------------
	//! @brief  名称
	//std::string engine::name()
	//	GTPの name コマンドに対応しています。
	//	プログラム(チーム)の名前を返して下さい
	//--------------------------------------
	virtual std::string name() 
	{
	    return ENTRY_NAME; 
	}
	
	//--------------------------------------
	//! @brief  バージョン
	//std::string engine::version()
	//	GTPの version コマンドに対応しています。
	//	プログラムのバージョンを返して下さい
	//--------------------------------------
	virtual std::string version() 
	{
		return CEDEC_VERSION; 
	}

	//--------------------------------------
	//! @brief  思考処理関数
	//	void engine::genmove(int& i, int& j, COLOR color)
	//	GTPの genmove に対応しています。
	//	現在の盤情報からcolor側の手を決定してi, jに設定して下さい。
	//	また↓playmoveを呼び出すなどして盤情報に反映して下さい
	//--------------------------------------
	virtual void genmove(int& i, int& j, COLOR color);

	//--------------------------------------
	//! @brief  １手すすめる
	//	void engine::playmove(int i, int j, COLOR color)
	//	GTPの playmove に対応しています。
	//	手情報がi, j, colorに入っていますので、盤情報に反映して下さい。
	//--------------------------------------
	virtual void playmove(int i, int j, COLOR color);

	//--------------------------------------
	//! @brief  １手戻す
	//void engine::undo()
	//	GTPの undo に対応しています。
	//	盤情報の最後の手ををundoして下さい。
	//--------------------------------------
	virtual void undo();

	//--------------------------------------
	//! @brief  そこに石を置けるかどうか
	//bool engine::legal_move(int i, int j, COLOR color)
	//	playmove呼び出しの直前に手の正否をチェックするために呼び出されます。
	//--------------------------------------
	virtual bool legal_move(int i, int j, COLOR color);
	  
	//--------------------------------------
	//! @brief  盤面取得
	//	COLOR engine::get_board(int i, int j)
	//	GTPの showboard コマンドに対応します。
	//	指定の座標の盤情報を返して下さい。
	//--------------------------------------
	virtual COLOR get_board(int i, int j);

	//--------------------------------------
	//! @brief  
	//	void engine::time_settings(int, int, int)
	//	GTPの time_settings に対応しています。
	//--------------------------------------
	virtual void time_settings(int main_time, int byo_yomi_time, int byo_yomi_stones)
	{
		//何もしない
	}
	
	//--------------------------------------
	//! @brief  
	//void engine::time_left(COLOR c, int t, int s)
	//GTPの time_left に対応しています。
	//tに残り時間(秒)が入っています。
	//--------------------------------------
	virtual void time_left(COLOR color, int time, int stones)
	{
		//何もしない
	}
	
	//--------------------------------------
	//! @brief  
	//void engine::compute_final_status()
	//	GTPの final_score に対応しています。
	//	試合終了後に呼出します。
	//	最終的な盤面情報を決定して下さい。
	//--------------------------------------
	virtual void compute_final_status();

	//--------------------------------------
	//! @brief  
	//COLOR engine::get_final_status(int i, int j)
	//	指定の座標の最終的な盤情報を返して下さい。
	//--------------------------------------
	virtual COLOR get_final_status(int i, int j);
};

};	//namespace my_go {
